--[[
    LootMonitor 3.00+
    Add Raid Instance Player Enter Announce.
    Use UNIT_FLAGS to flag who first open monster's corpse.
    Add LootLog Function.

    LootMonitor 2.10
    Reduce info.

    LootMonitor 2.06
    Fix Bug.

    2.05
    Update toc.
    
    2.04
    Fix: Sometime Announce.Index isn't legal.
    
    2.03
    Add: Always Announcer the Looter Checkbox

    2.02
    Fix: CHAT_MSG_ADDON: LMPX, "end CorpseGUID NumQuality" instead "end CorpseGUID Num".
    Get Correct Quality Rank.

    2.01
    What's New?
    1. Rewrite Sync Function, use table replace single variables to store group's states, make code more readable.
    2. Make Addon work correct in party mode. Caused by Party Sequence, Only Announce Loots when Party Leader Installed this Addon.
    3. Compatible with older Version. (1.38, 1.39 and 1.40)
    4. Make GUI more pleasing to the eye. Now, Announcer in group will mark as Crimson, player who do not have this Addon will display as Grey.
    5. Add Single Disable Mode. You will not announce anything if all group member except you do not installed this Addon.
]]

local AddonName, Addon = ...

-- 主Frame（不可见）
Addon.Frame = CreateFrame("Frame")
-- 设置界面（界面逻辑放置于GUI.lua）
Addon.SetWindow = {}
Addon.GroupState = {} -- 40个状态栏
Addon.Output = {} --输出窗口
-- 本地化相关
local L = Addon.L

-- 相关变量
local Frame = Addon.Frame
local Output = Addon.Output
Frame:Hide() -- 隐藏Frame，避免Update开销
-- 版本号（不再读取toc文件，/reload后版本号立刻生效）
Addon.Version = "3.04"
local Version = tonumber(Addon.Version)
-- Addon MSG Prefix
local PrefixLM = "LMPX"

-- WOW API加速
-- 普通函数
local UnitName = UnitName
local UnitGUID = UnitGUID
local UnitIsDead = UnitIsDead
local UnitIsGhost = UnitIsGhost
local UnitIsPlayer = UnitIsPlayer
local UnitInRaid = UnitInRaid
local UnitIsAFK = UnitIsAFK
local IsInGroup = IsInGroup
local IsInRaid = IsInRaid
local IsInInstance = IsInInstance
local UnitInBattleground = UnitInBattleground
local UnitIsGroupLeader = UnitIsGroupLeader
local UnitIsGroupAssistant = UnitIsGroupAssistant
local UnitIsConnected = UnitIsConnected
local UnitAffectingCombat = UnitAffectingCombat
local UnitPosition = UnitPosition
local UnitIsFriend = UnitIsFriend
local GetTime = GetTime
local GetNumGroupMembers = GetNumGroupMembers
local GetLootInfo = GetLootInfo
local GetLootSlotLink = GetLootSlotLink
local GetCVar = GetCVar
local GetRealZoneText = GetRealZoneText
local GetNumSavedInstances = GetNumSavedInstances
local GetSavedInstanceInfo = GetSavedInstanceInfo
local SendChatMessage = SendChatMessage
-- Lua方法加速
local date = date
local time = time
local t_insert = table.insert
local t_remove = table.remove
-- 同步相关函数
local SendAddonMessage = C_ChatInfo.SendAddonMessage
local IsAddonMessagePrefixRegistered = C_ChatInfo.IsAddonMessagePrefixRegistered
local RegisterAddonMessagePrefix = C_ChatInfo.RegisterAddonMessagePrefix
-- 常量
local PlayerName = ""
local ItemQuality = {
    [1] = L[" Common+ Item(s)"],
    [2] = L[" Uncommon+ Item(s)"],
    [3] = L[" Rare+ Item(s)"],
    [4] = L[" Epic+ Item(s)"],
}
local RaidInstanceID = {
	[249] = L["Onyxia's Lair"],
	[409] = L["Molten Core"],
	[469] = L["Blackwing Lair"],
    [531] = L["Temple of Ahn'Qiraj"],
    [533] = L["Naxxramas"],
    [309] = L["Zul'Gurub"],
    [509] = L["Ruins of Ahn'Qiraj"],
    [34] = "监狱",
} 
-- 控制变量
local Announce = {
    ["Rank"] = "NONE",
    ["Index"] = 0,
}
-- 数据库
-- 设置相关
Addon.Config = {
    ["OutputToRaidWarning"] = false,  -- 输出至Raid_Warning频道（向下兼容）
    ["Quality"] = 3, -- 默认通报物品等级：蓝色（3）
    ["EnteringCheck"] = true,
    ["ExpireDay"] = 7, -- 日志保存天数
}
local Config = Addon.Config
local LootLog = {}

-- 临时变量
local LooterInfo = {}
local ItemIndex = 0
local GroupMemberStates = {}
local GroupFlagsStates = {}
local PlayerInInstance = {}

-- 通用功能函数
-- 更新Table（合并）
function Addon:UpdateTable(Target, Source)
    for k, v in pairs(Source) do
        if type(v) == "table" then
            if type(Target[k]) == "table" then
                self:UpdateTable(Target[k], v)
            else
                Target[k] = self:UpdateTable({}, v)
            end
        elseif type(Target[k]) ~= "table" then
            Target[k] = v
        end
    end
    return Target
end

-- 操作函数
-- 新建LootLog表
function Addon:NewRaid()
    local TempRaid = {
		["Date"] = date("%Y-%m-%d"),
        ["InstanceID"] = nil,
        ["InstanceName"] = nil,
        ["SavedID"] = nil,
        ["FirstPlayer"] = nil,
        ["LootTable"] = {},
    }
    return TempRaid
end
-- 新建Loot表
function Addon:NewLoots()
    local TempMob = {
		["Date"] = date("%Y-%m-%d"),
        ["Time"] = date("%H:%M:%S"),
        ["Name"] = nil,
        ["GUID"] = nil,
        ["Looter"] = nil,
        ["Loots"] = {},
    }
    return TempMob
end
-- 升级
function Addon:DoUpdateCheck(VersionString)
    if tonumber(VersionString) > Version then
        print(L["<|cFFFF4500LM|r>Need Update your |cFFFF4500LootMonitor|r to newest one!"])
        print(L["|cFF4169E1Download Link|r https://www.curseforge.com/wow/addons/lootmonitor"])
    end
end
-- 检查
function Addon:DoCheck()
    if UnitInBattleground("player") then -- 战场禁用
        return
    end
    if IsInRaid() then
        SendAddonMessage(PrefixLM, "check " .. Addon.Version, "raid")
    elseif IsInGroup() then
        SendAddonMessage(PrefixLM, "check " .. Addon.Version, "party")
    end
    for i = 1, 40 do
        Addon.GroupState[i]:SetText("")
    end
end
-- 答复
function Addon:DoReply()
    if UnitInBattleground("player") then -- 战场禁用
        return
    end
    if IsInRaid() then
        SendAddonMessage(PrefixLM, "reply " .. Addon.Version, "raid")
    elseif IsInGroup() then
        SendAddonMessage(PrefixLM, "reply " .. Addon.Version, "party")
    end
end
-- 显示状态
function Addon:DoShowState()
    if next(GroupMemberStates) == nil then
        return
    end
    local AnnouncerColorPrefix = "|cFFDC143C"
    local NoAddonColorPrefix = "|cFF606060"
    local ColorSuffix = "|r"
    for i = 1, #GroupMemberStates do
        if GroupMemberStates[i] then
            local msg = GroupMemberStates[i].Name
            if i == Announce.Index then
                msg = AnnouncerColorPrefix..msg..ColorSuffix.." (v"..tostring(GroupMemberStates[i].Version)..")"
            elseif GroupMemberStates[i].Version == 0 then
                msg = NoAddonColorPrefix..msg.." ("..L["NoAddon"]..")"..ColorSuffix
            else
                msg = msg.." (v"..tostring(GroupMemberStates[i].Version)..")"
            end
            Addon.GroupState[i]:SetText(msg)
            -- Addon.GroupState[i]:Show()
        end
    end
end
-- 初始化团队成员状态
function Addon:InitGroupFlags()
    if UnitInBattleground("player") then -- 战场禁用
        return
    end
    GroupFlagsStates = {}
    if IsInRaid() then
        for i = 1, 40 do
            local u = "raid"..i
            if (UnitName(u)) then
                GroupFlagsStates[u] = {
                    ["Name"] = (UnitName(u)),
                    ["Connected"] = UnitIsConnected(u),
                    ["DeadOrGhost"] = UnitIsDead(u) and "Dead" or (UnitIsGhost(u) and "Ghost" or "Alive"),
                    ["InCombat"] = UnitAffectingCombat(u),
                    ["IsAFK"] = UnitIsAFK(u)
                }
            else
                GroupFlagsStates[u] = nil
            end
        end
    elseif IsInGroup() then
        GroupFlagsStates["player"] = {
            ["Name"] = (UnitName("player")),
            ["Connected"] = true,
            ["DeadOrGhost"] = UnitIsDead("player") and "Dead" or (UnitIsGhost("player") and "Ghost" or "Alive"),
            ["InCombat"] = UnitAffectingCombat("player"),
            ["IsAFK"] = UnitIsAFK("player")
        }
        for i = 1, 4 do
            local u = "party"..i
            if (UnitName(u)) then
                GroupFlagsStates[u] = {
                    ["Name"] = (UnitName(u)),
                    ["Connected"] = UnitIsConnected(u),
                    ["DeadOrGhost"] = UnitIsDead(u) and "Dead" or (UnitIsGhost(u) and "Ghost" or "Alive"),
                    ["InCombat"] = UnitAffectingCombat(u),
                    ["IsAFK"] = UnitIsAFK(u)
                }
            end
        end
    else
        GroupFlagsStates = {}
    end
end
-- 初始化团队信息
function Addon:InitGroupInfo()
    if UnitInBattleground("player") then -- 战场禁用
        return
    end
    GroupMemberStates = {}
    if IsInRaid() then -- 初始化Raid团队信息
        for i = 1, 40 do
            local u = "raid"..i
            local n = UnitName(u)
            if UnitName(u) then
                local state = {
                    ["Index"] = u,
                    ["Name"] = n,
                    ["Rank"] = "NONE",
                    ["LootMonitor"] = false,
                    ["Version"] = 0,
                }
                t_insert(GroupMemberStates, state)
            else
                break
            end
        end
    elseif IsInGroup() then -- 初始化队伍信息（Player放在最前）
        local PlayerRank = "NONE"
        if UnitIsGroupLeader("player") then
            PlayerRank = "LEADER"
            Announce.Rank = "LEADER"
            Announce.Index = 1
        end
        local playerstate = {
            ["Index"] = "player",
            ["Name"] = PlayerName,
            ["Rank"] = PlayerRank,
            ["LootMonitor"] = true,
            ["Version"] = Version,
        }
        t_insert(GroupMemberStates, playerstate)
        for i = 1, 4 do
            local u = "party"..i
            local n = UnitName(u)
            if UnitName(u) then
                local state = {
                    ["Index"] = u,
                    ["Name"] = n,
                    ["Rank"] = "NONE",
                    ["LootMonitor"] = false,
                    ["Version"] = 0,
                }
                t_insert(GroupMemberStates, state)
            else
                break
            end
        end
    end
end
-- 更新团队成员信息
function Addon:UpdateGroupInfo(SenderName, VersionString)
    if IsInRaid() then
        local SenderIndex = UnitInRaid(SenderName)
        if SenderIndex then
            local SenderRank = "NONE"
            if UnitIsGroupLeader(SenderName) then -- 信息发送者是团长
                SenderRank = "LEADER"
                Announce.Rank = "LEADER"
                Announce.Index = SenderIndex
            elseif UnitIsGroupAssistant(SenderName) then -- 信息发送者是助理
                SenderRank = "ASSISTANT"
                if Announce.Rank == "NONE" then
                    Announce.Rank = "ASSISTANT"
                    Announce.Index = SenderIndex
                elseif Announce.Rank == "ASSISTANT" then
                    if SenderIndex < Announce.Index then
                        Announce.Index = SenderIndex
                    end
                end
            else -- 普通团员
                if Announce.Rank == "NONE" then
                    if Announce.Index == 0 then
                        Announce.Index = SenderIndex
                    elseif SenderIndex < Announce.Index then
                        Announce.Index = SenderIndex
                    end
                end
            end
            local SenderVersion = tonumber(VersionString)
            GroupMemberStates[SenderIndex] = {
                ["Index"] = "raid"..SenderIndex,
                ["Name"] = SenderName,
                ["Rank"] = SenderRank,
                ["LootMonitor"] = true,
                ["Version"] = SenderVersion,
            }
        end
    elseif IsInGroup() then
        local SenderRank = "NONE"
        if UnitIsGroupLeader(SenderName) then
            SenderRank = "LEADER"
        end
        local SenderVersion = tonumber(VersionString)
        for i = 1, #GroupMemberStates do
            if GroupMemberStates[i].Name == SenderName then
                GroupMemberStates[i].Rank = SenderRank
                GroupMemberStates[i].LootMonitor = true
                GroupMemberStates[i].Version = SenderVersion
                if SenderRank == "LEADER" then
                    Announce.Rank = "LEADER"
                    Announce.Index = i
                end
                break
            end
        end
    end
end

-- 输出LootLog
function Addon:PrintLootLog()
	Output.title:SetText(L["Loot Logs"])
	Output.background:Show()
	Output.export:GetParent():Show()
	-- 清理不合法LootLog
	for i = #LootLog, 1, -1 do
		if not LootLog[i].Date then
			t_remove(LootLog, i)
		end
	end
	if #LootLog == 0 then
		Output.export:SetText(L["<|cFFBA55D3LootMonitor|r>No Loot Log!"].."\n")
		return
    end
    local msg = ""
    -- todo: display lootlogs
    for i = 1, #LootLog do
        msg = msg .. string.format(L["[|cFFFFFF00%s|r] |cFFFF69B4Raid Instance:|r |cFFDC143C%s|r\n    |cFFF0F000First Entered Player:|r |cFF00FF00%s|r\n    |cFF9932CCInstance ID:|r |cFFEE82EE%s|r\n"], LootLog[i].Date, LootLog[i].InstanceName, LootLog[i].FirstPlayer, LootLog[i].SavedID or L["UNKNOWN"])
        if #LootLog[i].LootTable > 0 then
            for j = 1, #LootLog[i].LootTable do
                msg = msg .. string.format("        |cFF00FF00%s|r -> |cFFDC143C%s|r\n", LootLog[i].LootTable[j].Looter, LootLog[i].LootTable[j].Name)
                for k = 1, #LootLog[i].LootTable[j].Loots do
                    msg = msg .. "          [" .. k .. "] " .. LootLog[i].LootTable[j].Loots[k] .. "\n"
                end
            end
        end
    end

    Output.export:SetText(msg)
end
-- 清空LootLog
function Addon:CleanLootLogs()
    if #LootLog > 0 then
        for i = #LootLog, 1, -1 do
            t_remove(LootLog, i)
        end
    end
    self:PrintLootLog()
end

-- 注册事件
Frame:RegisterEvent("ADDON_LOADED")
Frame:RegisterEvent("PLAYER_LOGOUT")
Frame:RegisterEvent("LOOT_OPENED")
Frame:RegisterEvent("CHAT_MSG_ADDON")
-- Frame:RegisterEvent("ZONE_CHANGED")
Frame:RegisterEvent("GROUP_ROSTER_UPDATE")
Frame:RegisterEvent("UNIT_FLAGS")

-- New本检查
do
    local LastScan = 0

    Frame:SetScript("OnUpdate", function(self, lastupdate)
        if not Config.EnteringCheck or not IsInRaid() then
            Frame:Hide()
            return
        end
        local NowScan = math.floor(GetTime())
        if NowScan == LastScan then
            return
        end
        LastScan = NowScan
        for i = 1, GetNumGroupMembers() do
            local InstanceID = select(4, UnitPosition("raid"..i))
            if RaidInstanceID[InstanceID] then
                if not PlayerInInstance[InstanceID] then
                    PlayerInInstance[InstanceID] = {}
                    local PlayerName = (UnitName("raid"..i))
                    t_insert(PlayerInInstance[InstanceID], PlayerName)
                else
                    local PlayerName = (UnitName("raid"..i))
                    t_insert(PlayerInInstance[InstanceID], PlayerName)
                end
            end
        end
        if (next(PlayerInInstance)) then
            for k in pairs(PlayerInInstance) do
                if #PlayerInInstance[k] == 1 then
                    local msg = string.format(L["<LM>The first player who entered <%s> Raid Instance is <%s>."], RaidInstanceID[k], PlayerInInstance[k][1])
                    SendAddonMessage(PrefixLM, "enter "..msg, "raid")
                    Frame:Hide()
                elseif #PlayerInInstance[k] > 1 then
                    Frame:Hide()
                end
            end
        end
    end)
end

-- 事件分配
Frame:SetScript("OnEvent", function(self, event, ...)
    if type(self[event]) == "function" then
        return self[event](self, ...)
    end
end)

-- 事件处理
-- 加载处理
function Frame:ADDON_LOADED(Name)
    if Name ~= AddonName then
        return
    end
    -- 反注册ADDON_LOADED事件
    self:UnregisterEvent("ADDON_LOADED")
    -- 读取存档资料，如果没有LootMonitorDB.Config则新建
    if not LootMonitorDB then
        LootMonitorDB = {
            ["Config"] = {},
            ["LootLog"] = {},
        }
    end
    if not LootMonitorDB.Config then
        LootMonitorDB.Config = {}
    end
    if not LootMonitorDB.LootLog then
        LootMonitorDB.LootLog = {}
    end
    -- 用存档文件数据更新Config表
    Addon:UpdateTable(Config, LootMonitorDB.Config)
    Addon:UpdateTable(LootLog, LootMonitorDB.LootLog)
    -- 注册Prefix
    if not IsAddonMessagePrefixRegistered(PrefixLM) then
        RegisterAddonMessagePrefix(PrefixLM)
    end
    -- 反和谐相关
    if GetCVar("portal") == "CN" then
        ConsoleExec("portal KR")
        ConsoleExec("profanityFilter 0")
        ConsoleExec("overrideArchive 0")
    end
    -- 初始化常量
    PlayerName = UnitName("player")
    -- 初始化设定窗口
    Addon.SetWindow:Initialize()
   	-- 初始化Output窗口
	Addon.Output:Initialize()
    -- 初始化团队成员版本信息表
    Addon:InitGroupInfo()
    -- 初始化团队成员状态表
    Addon:InitGroupFlags()
    if Config.EnteringCheck then
        Frame:Show()
    else
        Frame:Hide()
    end
	-- 清理过期TradeLog
	local Today = {}
	Today.year, Today.month, Today.day = strsplit("-", date("%Y-%m-%d"))
	local LogDay = {}
	if #LootLog > 0 then
		for i = #LootLog, 1, -1 do
			if LootLog[i].Date then
				LogDay.year, LogDay.month, LogDay.day = strsplit("-", LootLog[i].Date)
			else
				t_remove(LootLog, i)
			end
			if (time(Today) - time(LogDay)) / (3600 * 24) > Config.ExpireDay then
				t_remove(LootLog, i)
			end
		end
	end
end
-- 退出处理（包括/reload）
function Frame:PLAYER_LOGOUT()
    -- 清空LootMonitorDB
    LootMonitorDB = {
        ["Config"] = {},
        ["LootLog"] = {},
    }
    -- 用当前Config覆盖LootMonitorDB
    Addon:UpdateTable(LootMonitorDB.Config, Config)
    Addon:UpdateTable(LootMonitorDB.LootLog, LootLog)
    LooterInfo = nil
end
-- ADDON频道信息处理
function Frame:CHAT_MSG_ADDON(...)
    if UnitInBattleground("player") then -- 战场禁用
        return
    end
    if not IsInGroup() then -- 非组队情况下退出
        return
    end
    -- 获取事件信息
    local arg = {...}
    -- 团队和小队处理模式一致
    if arg[1] == PrefixLM then
        -- 切分信息，以空格为界
        local CMD, REST = arg[2]:match("(%S+)%s+(.-)$")
        -- 确定输出频道
        local OutputChannel = "party"
        if IsInRaid() then
            OutputChannel = "raid"
            if UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") then
                if Config.OutputToRaidWarning then
                    OutputChannel = "raid_warning"
                end
            end
        end
        if CMD:lower() == "check" then -- 收到检查命令，返回reply信息
            -- 进行答复
            Addon:DoReply()
            -- 比较版本号，提示更新
            if REST then
                Addon:DoUpdateCheck(REST)
            end
        elseif CMD:lower() == "reply" or CMD:lower() == "start" then
            if arg[5] and REST then
                Addon:UpdateGroupInfo(arg[5], REST) -- 更新团队插件信息
                -- 比较版本号，提示更新
                if REST then
                    Addon:DoUpdateCheck(REST)
                end
            end
        elseif CMD:lower() == "target" then
            local TargetName, TargetGUID = REST:match("(%S+)%s(.-)$")
            if TargetGUID and Announce.Index ~= 0 then
                if not LooterInfo[TargetGUID] then
                    LooterInfo[TargetGUID] = {
                        ["LooterName"] = arg[5],
                        ["Sender"] = arg[5],
                        ["LootAnnounced"] = false,
                    }
                else
                    LooterInfo[TargetGUID].Sender = arg[5]
                end
                if not LooterInfo[TargetGUID].LootAnnounced then
                    LooterInfo[TargetGUID].LootAnnounced = true
                    ItemIndex = 1
                    if GroupMemberStates[Announce.Index] and GroupMemberStates[Announce.Index].Name == UnitName("player") and (select(2, IsInInstance()) == "raid" or select(2, IsInInstance()) == "party") then
                        SendChatMessage(string.format(L["<LM><%s> has Looted <%s>'s Corpse"], LooterInfo[TargetGUID].LooterName, TargetName), OutputChannel)
                    elseif GroupMemberStates[Announce.Index] and GroupMemberStates[Announce.Index].Name == UnitName("player") then
                        SendChatMessage(string.format(L["<LM><%s>'s Corpse Opened"], TargetName), OutputChannel)
                    end
                    -- 记录Loot信息
                    local InstanceID = select(4, UnitPosition("player"))
                    if #LootLog >= 1 then
                        if LootLog[#LootLog].InstanceID == InstanceID then
                            local Loots = Addon:NewLoots()
                            Loots.Name = TargetName
                            Loots.GUID = TargetGUID
                            Loots.Looter = LooterInfo[TargetGUID].LooterName
                            t_insert(LootLog[#LootLog].LootTable, Loots)
                            if not LootLog[#LootLog].SavedID then
                                local SavedRaidID = nil
                                for i = 1, GetNumSavedInstances() do
                                    if (GetSavedInstanceInfo(i)) == GetRealZoneText() then
                                        SavedRaidID = (select(2, GetSavedInstanceInfo(i)))
                                        break
                                    end
                                end
                                LootLog[#LootLog].SavedID = SavedRaidID
                            end
                        elseif RaidInstanceID[InstanceID] then
                            local SavedRaidID = nil
                            for i = 1, GetNumSavedInstances() do
                                if (GetSavedInstanceInfo(i)) == GetRealZoneText() then
                                    SavedRaidID = (select(2, GetSavedInstanceInfo(i)))
                                    break
                                end
                            end
                            local TempRaid = Addon:NewRaid()
                            TempRaid.InstanceID = InstanceID
                            TempRaid.InstanceName = RaidInstanceID[InstanceID]
                            TempRaid.SavedID = SavedRaidID
                            TempRaid.FirstPlayer = RaidInstanceID[TempRaid.InstanceID] and PlayerInInstance[TempRaid.InstanceID][1] or L["UNKNOWN"]
                            t_insert(LootLog, TempRaid)
                            local Loots = Addon:NewLoots()
                            Loots.Name = TargetName
                            Loots.GUID = TargetGUID
                            Loots.Looter = LooterInfo[TargetGUID].LooterName
                            t_insert(LootLog[#LootLog].LootTable, Loots)
                        end
                    elseif RaidInstanceID[InstanceID] then
                        local SavedRaidID = nil
                        for i = 1, GetNumSavedInstances() do
                            if (GetSavedInstanceInfo(i)) == GetRealZoneText() then
                                SavedRaidID = (select(2,GetSavedInstanceInfo(i)))
                                break
                            end
                        end
                        local TempRaid = Addon:NewRaid()
                        TempRaid.InstanceID = InstanceID
                        TempRaid.InstanceName = RaidInstanceID[InstanceID]
                        TempRaid.SavedID = SavedRaidID
                        TempRaid.FirstPlayer = RaidInstanceID[TempRaid.InstanceID] and PlayerInInstance[TempRaid.InstanceID][1] or L["UNKNOWN"]
                        t_insert(LootLog, TempRaid)
                        local Loots = Addon:NewLoots()
                        Loots.Name = TargetName
                        Loots.GUID = TargetGUID
                        Loots.Looter = LooterInfo[TargetGUID].LooterName
                        t_insert(LootLog[#LootLog].LootTable, Loots)
                    end
                end
            end
        elseif CMD:lower() == "loot" and ItemIndex ~= 0 then
            local TargetGUID, LootLink = REST:match("(%S+)%s+(.-)$")
            if LooterInfo[TargetGUID].Sender == arg[5] and Announce.Index ~= 0 then
                if GroupMemberStates[Announce.Index] and GroupMemberStates[Announce.Index].Name == PlayerName then
                    SendChatMessage(string.format("%d: %s", ItemIndex, LootLink), OutputChannel)
                end
                LootLog[#LootLog].LootTable[#LootLog[#LootLog].LootTable].Loots[ItemIndex] = LootLink
                ItemIndex = ItemIndex + 1
            end
        elseif CMD:lower() == "end" and ItemIndex ~= 0 then
            local TargetGUID, LootsNum = REST:match("(%S+)%s+(.-)$")
            if LooterInfo[TargetGUID].Sender == arg[5] and Announce.Index ~= 0 then
                if GroupMemberStates[Announce.Index] and GroupMemberStates[Announce.Index].Name == PlayerName then
                    SendChatMessage(L["<LM>Total "]..LootsNum, OutputChannel)
                end
                ItemIndex = 0
            end
        elseif CMD:lower() == "enter" then
            if GroupMemberStates[Announce.Index] and GroupMemberStates[Announce.Index].Name == PlayerName then
                SendChatMessage(REST, "raid")
            end
        end
    end
end
-- 队伍调整后重新初始化团队信息，并发送状态
function Frame:GROUP_ROSTER_UPDATE()
    if UnitInBattleground("player") then -- 战场禁用
        return
    end
    -- 初始化Announce表
    Announce.Rank = "NONE"
    Announce.Index = 0
    -- 初始化GroupMemberStates表
    Addon:InitGroupInfo()
    -- 初始化团队成员状态表
    Addon:InitGroupFlags()
    -- 同步版本信息
    if IsInRaid() then
        SendAddonMessage(PrefixLM, "reply " .. Addon.Version, "raid")
        if not Frame:IsShown() then
            Frame:Show()
        end
    elseif IsInGroup() then
        SendAddonMessage(PrefixLM, "reply " .. Addon.Version, "party")
        Frame:Hide()
    else
        Frame:Hide()
        PlayerInInstance = {}
    end
end
-- 处理Flags事件
function Frame:UNIT_FLAGS(Unit)
    if UnitInBattleground("player") then -- 战场禁用
        return
    end
    if not UnitIsPlayer(Unit) then -- 第一层过滤网，过滤非玩家Unit（Target、Pet）
        return
    end
    if UnitIsPlayer(Unit.."target") then -- 第二层过滤网，按目标过滤（目标是玩家）
        return
    end
    if not UnitIsDead(Unit.."target") then -- 第三层过滤网，按目标过滤（目标未死亡）
        return
    end
    if IsInRaid() and string.find(Unit, "raid") and UnitIsFriend(Unit, "player") then
        -- print(Unit)
        local Connected = UnitIsConnected(Unit)
        local DeadOrGhost = UnitIsDead(Unit) and "Dead" or (UnitIsGhost(Unit) and "Ghost" or "Alive")
        local InCombat = UnitAffectingCombat(Unit)
        local IsAFK = UnitIsAFK(Unit)
        if GroupFlagsStates[Unit].Connected == Connected and GroupFlagsStates[Unit].DeadOrGhost == DeadOrGhost and GroupFlagsStates[Unit].InCombat == InCombat and GroupFlagsStates[Unit].IsAFK == IsAFK and UnitIsDead(Unit.."target") then
            local TargetGUID = UnitGUID(Unit.."target")
            if not LooterInfo[TargetGUID] then
                LooterInfo[TargetGUID] = {
                    ["LooterName"] = GroupFlagsStates[Unit].Name,
                    ["Sender"] = nil,
                    ["LootAnnounced"] = false,
                }
                -- print("Player = " ..(UnitName(Unit)))
                -- print("Connected:"..tostring(GroupFlagsStates[Unit].Connected))
                -- print(GroupFlagsStates[Unit].DeadOrGhost)
                -- print("Combat:"..tostring(GroupFlagsStates[Unit].InCombat))
                -- print("AFK:"..tostring(GroupFlagsStates[Unit].IsAFK))
            end
        end
        if GroupFlagsStates[Unit].Connected ~= Connected then
            GroupFlagsStates[Unit].Connected = Connected
        elseif GroupFlagsStates[Unit].DeadOrGhost ~= DeadOrGhost then
            GroupFlagsStates[Unit].DeadOrGhost = DeadOrGhost
        elseif GroupFlagsStates[Unit].InCombat ~= InCombat then
            GroupFlagsStates[Unit].InCombat = InCombat
        elseif GroupFlagsStates[Unit].IsAFK ~= IsAFK then
            GroupFlagsStates[Unit].IsAFK = IsAFK
        end
    elseif IsInGroup() and not IsInRaid() and (Unit == "player" or string.find(Unit, "party")) and UnitIsFriend(Unit, "player") then
        local Connected = UnitIsConnected(Unit)
        local DeadOrGhost = UnitIsDead(Unit) and "Dead" or (UnitIsGhost(Unit) and "Ghost" or "Alive")
        local InCombat = UnitAffectingCombat(Unit)
        local IsAFK = UnitIsAFK(Unit)
        if GroupFlagsStates[Unit] and GroupFlagsStates[Unit].Connected == Connected and GroupFlagsStates[Unit].DeadOrGhost == DeadOrGhost and GroupFlagsStates[Unit].InCombat == InCombat and GroupFlagsStates[Unit].IsAFK == IsAFK and UnitIsDead(Unit.."target") then
            local TargetGUID = UnitGUID(Unit.."target")
            if not LooterInfo[TargetGUID] then
                LooterInfo[TargetGUID] = {
                    ["LooterName"] = GroupFlagsStates[Unit].Name,
                    ["Sender"] = nil,
                    ["LootAnnounced"] = false,
            }
            end
        end
        if GroupFlagsStates[Unit].Connected ~= Connected then
            GroupFlagsStates[Unit].Connected = Connected
        elseif GroupFlagsStates[Unit].DeadOrGhost ~= DeadOrGhost then
            GroupFlagsStates[Unit].DeadOrGhost = DeadOrGhost
        elseif GroupFlagsStates[Unit].InCombat ~= InCombat then
            GroupFlagsStates[Unit].InCombat = InCombat
        elseif GroupFlagsStates[Unit].IsAFK ~= IsAFK then
            GroupFlagsStates[Unit].IsAFK = IsAFK
        end
    end
end
-- 发送Loot信息
function Frame:LOOT_OPENED(...)
    if UnitInBattleground("player") then -- 战场禁用
        return
    end
    if not IsInGroup() then -- 非组队状态禁用
        return
    end
    if not UnitIsDead("target") then -- 目标是否已死亡
        return
    end
    local loots = GetLootInfo() -- 获取Loot信息
    local msg = {} -- 初始化需要发送的Loot信息表
    -- 设置发送频道
    local channel = "party"
    if IsInRaid() then
        channel = "raid"
    end
    -- 根据设定的物品等级，填充物品信息表
    for i = 1, #loots do
        if loots[i] then
            if loots[i].quantity ~= 0 then
                if loots[i].quality >= Config.Quality then
                    t_insert(msg, GetLootSlotLink(i))
                end
            end
        end
    end
    -- 发送物品信息
    if #msg > 0 then
        SendAddonMessage(PrefixLM, "target " .. UnitName("target") .. " " .. UnitGUID("target"), channel)
        for i = 1, #msg do
            SendAddonMessage(PrefixLM, "loot " .. UnitGUID("target") .. " " .. msg[i], channel)
        end
        SendAddonMessage(PrefixLM, "end " .. UnitGUID("target") .. " " .. tostring(#msg)..ItemQuality[Config.Quality], channel)
    end
end



-- 注册slash命令
SLASH_LMC1 = "/lootmonitor"
SLASH_LMC2 = "/lm"
-- 处理slash命令
SlashCmdList["LMC"] = function(Command)
    if Command:lower() == "gui" then
        if Addon.SetWindow:IsShown() then
            Addon.SetWindow:Hide()
        else
            Addon.SetWindow:Show()
        end
    elseif Command:lower() == "lootlog" or Command:lower() == "ll" then
        Addon:PrintLootLog()
    else
        print(L["|cFFBA55D3LootMonitor|r Tips: Use |cFF00BFFF/lootmonitor|r |cFFFF4500gui|r or |cFF00BFFF/lm|r |cFFFF4500gui|r open GUI, Use |cFF00BFFF/lootmonitor|r |cFFFF9000lootlog|r or |cFF00BFFF/lm|r |cFFFF9000ll|r open Loots Logs."])
    end
end